@@ -33,7 +33,7 @@ our %EXPORT_TAGS =
      env   => [qw(env_set env_push env_pop)],
      pack  => [qw(packsize packFilterFetch packFilterStore)],
      math  => [qw($LOG2 log2 min2 max2 lmax lmin lsum)],
-     list  => [qw(luniq xluniq)],
+     list  => [qw(luniq sluniq xluniq)],
      regex => [qw(regex)],
      html  => [qw(htmlesc)],
      time  => [qw(s2hms s2timestr timestamp)],
@@ -418,12 +418,19 @@ sub lsum {
 ## Functions: lists
 
 ## \@l_uniq = luniq(\@l)
-##  + returns unique defined elements of @l
+##  + returns unique defined elements of @l; @l need not be sorted
 sub luniq {
   my ($tmp);
   return [map {defined($tmp) && $tmp eq $_ ? qw() : ($tmp=$_)} sort grep {defined($_)} @{$_[0]//[]}];
 }
 
+## \@l_sorted_uniq = sluniq(\@l_sorted)
+##  + returns unique defined elements of pre-sorted @l
+sub sluniq {
+  my ($tmp);
+  return [map {defined($tmp) && $tmp eq $_ ? qw() : ($tmp=$_)} grep {defined($_)} @{$_[0]//[]}];
+}
+
 ## \@l_uniq = xluniq(\@l,\&keyfunc)
 ##  + returns elements of @l with unique defined keys according to \&keyfunc (default=\&overload::StrVal)
 sub xluniq {
@@ -512,7 +519,7 @@ sub s2timestr {
 ## $rfc_timestamp = PACAKGE->timestamp($time)
 sub timestamp {
   shift if (UNIVERSAL::isa($_[0],__PACKAGE__));
-  return POSIX::strftime("%Y-%m-%dT%H:%M:%SZ", gmtime(@_ ? $_[0] : qw()));
+  return POSIX::strftime("%Y-%m-%dT%H:%M:%SZ", (@_ ? gmtime($_[0]) : gmtime()));
 }
 
 ##==============================================================================
